import json
import unittest

from django.contrib.auth.models import User
from django.test import Client


def get_user(username):
    try:
        user = User.objects.get(username=username)
        return user
    except Exception:
        user = User.objects.create_user(username, password='pass')
        return user


def get_user_pass(username):
    return 'pass'


class EMSWhenIWorkTest(unittest.TestCase):
    def setUp(self):
        # Every test needs a client.
        self.client = Client()

    def set_user(self, username):
        get_user(username)
        self.client.login(username=username,
                          password=get_user_pass(username))

    def test_serviceorders(self):
        self.set_user('javerage')

        # Issue a GET request.
        response = self.client.get('/serviceorders')

        # Check that the response is 200 OK.
        self.assertEqual(response.status_code, 200)

    def test_api_schedule(self):
        # Issue a GET request.
        response = self.client.get(
            '/api/v1/schedule/?StartDate=2016-01-02&EndDate=2016-01-02')

        # Check that the response is 200 OK.
        self.assertEqual(response.status_code, 200)

        data = json.loads(response.content)

        self.assertEquals(len(data), 3)



from django.conf.urls import include, url
from django.views.defaults import page_not_found

from .views import serviceorders
from .views.api.booking import Booking
from .views.api.building import Building
from .views.api.schedule import Schedule
from .views.api.serviceorder import ServiceOrder
from .views.api.shift import Shift
from .views.api.status import Status

urlpatterns = [
    url(r'^$', page_not_found, {'exception': 'none'}, name='ems_wheniwork'),

    url(r'^serviceorders/?$', serviceorders.index, name='serviceorders'),

    # Basic EMS data
    url(r'^api/v1/booking/$', Booking().run),
    url(r'^api/v1/building/$', Building().run),
    url(r'^api/v1/serviceorder/$', ServiceOrder().run),
    url(r'^api/v1/status/$', Status().run),

    url(r'^api/v1/shift/(?P<shift_id>\d+)?$', Shift().run),
    url(r'^api/v1/schedule/$', Schedule().run),
]



import logging

from django.forms.models import model_to_dict
from ems_client.service import Service

from . import RESTDispatch

logger = logging.getLogger(__name__)


class Building(RESTDispatch):
    def __init__(self):
        self._service_api = Service()
        self._audit_log = logging.getLogger('audit')

    def GET(self, request, **kwargs):
        buildings = self._service_api.get_buildings()

        buildings2 = []
        for building in buildings:
            buildings2.append(model_to_dict(building))

        return self.json_response(buildings2)



import logging

from django.forms.models import model_to_dict
from ems_client.service import Service

from . import RESTDispatch

logger = logging.getLogger(__name__)


class Status(RESTDispatch):
    def __init__(self):
        self._service_api = Service()
        self._audit_log = logging.getLogger('audit')

    def GET(self, request, **kwargs):
        statuses = self._service_api.get_statuses()

        statuses2 = []
        for status in statuses:
            status2 = model_to_dict(status)
            for key in status2.keys():
                if hasattr(status, 'get_%s_display' % key):
                    status2['dv_%s' % key] = getattr(
                        status, 'get_%s_display' % key)()
            statuses2.append(status2)

        return self.json_response(statuses2)



import logging

from django.forms.models import model_to_dict
from ems_client.service import Service

from . import RESTDispatch

logger = logging.getLogger(__name__)


class ServiceOrder(RESTDispatch):
    def __init__(self):
        self._service_api = Service()
        self._audit_log = logging.getLogger('audit')

    def GET(self, request, **kwargs):
        serviceorders = self._service_api.get_service_order_details(
            start_date=request.GET.get('StartDate'),
            end_date=request.GET.get('EndDate'))

        serviceorders2 = []
        for serviceorder in serviceorders:
            serviceorders2.append(model_to_dict(serviceorder))

        return self.json_response(serviceorders2)



import logging

from django.forms.models import model_to_dict
from ems_client.service import Service

from . import RESTDispatch

logger = logging.getLogger(__name__)


class Booking(RESTDispatch):
    def __init__(self):
        self._service_api = Service()
        self._audit_log = logging.getLogger('audit')

    def GET(self, request, **kwargs):
        params = {
            "start_date": request.GET.get('StartDate'),
            "end_date": request.GET.get('EndDate'),
            "statuses": request.GET.get('Statuses'),
        }
        if params['statuses']:
            params['statuses'] = params['statuses'].split(',')

        bookings = self._service_api.get_bookings(**params)

        bookings2 = []
        for booking in bookings:
            booking2 = model_to_dict(booking)
            for key in booking2.keys():
                if hasattr(booking, 'get_%s_display' % key):
                    booking2['dv_%s' % key] = getattr(
                        booking, 'get_%s_display' % key)()
            bookings2.append(booking2)

        return self.json_response(bookings2)



import csv
import json

from django.http import HttpResponse
from django.views.decorators.cache import never_cache


class RESTDispatch(object):
    """ Handles passing on the request to the correct view method
        based on the request type.
    """

    @never_cache
    def run(self, *args, **named_args):
        request = args[0]

        if "GET" == request.method:
            if hasattr(self, "GET"):
                return self.GET(*args, **named_args)
            else:
                return self.invalid_method(*args, **named_args)
        elif "POST" == request.method:
            if hasattr(self, "POST"):
                return self.POST(*args, **named_args)
            else:
                return self.invalid_method(*args, **named_args)
        elif "PUT" == request.method:
            if hasattr(self, "PUT"):
                return self.PUT(*args, **named_args)
            else:
                return self.invalid_method(*args, **named_args)
        elif "DELETE" == request.method:
            if hasattr(self, "DELETE"):
                return self.DELETE(*args, **named_args)
            else:
                return self.invalid_method(*args, **named_args)
        elif "PATCH" == request.method:
            if hasattr(self, "PATCH"):
                return self.PATCH(*args, **named_args)
            else:
                return self.invalid_method(*args, **named_args)
        else:
            return self.invalid_method(*args, **named_args)

    def invalid_method(self, *args, **named_args):
        return HttpResponse("Method not allowed", status=405)

    def data_exception_response(self, exception):
        message = "Error: %s" % exception.msg.capitalize()
        return HttpResponse(json.dumps({"error": message}),
                            status=exception.status,
                            content_type="application/json")

    def error_response(self, status, message="", content={}):
        content["error"] = message
        return HttpResponse(json.dumps(content),
                            status=status,
                            content_type="application/json")

    def _json_dump_default(self, obj):
        if hasattr(obj, 'isoformat'):
            return obj.isoformat()

        raise TypeError(
            "Unserializable object {} of type {}".format(obj, type(obj))
        )

    def json_response(self, content="", status=200):
        return HttpResponse(
            json.dumps(content, default=self._json_dump_default, indent=4),
            status=status,
            content_type="application/json"
        )

    def tsv_response(self, content="", fields=[], status=200):
        # Create the HttpResponse object with the appropriate CSV header.
        response = HttpResponse(content_type='text/tab-separated-values')

        if not len(fields):
            fields = content[0].keys()

        writer = csv.DictWriter(
            response,
            fieldnames=fields, extrasaction='ignore', dialect='excel-tab'
        )

        writer.writerows(content)

        return response

    def xml_response(self, content="", status=200):
        return HttpResponse(content,
                            status=status,
                            content_type="application/xml")



import json
import logging
import re

from uw_wheniwork.shifts import Shifts

from .exceptions import InvalidParamException
from . import RESTDispatch

logger = logging.getLogger(__name__)


class Shift(RESTDispatch):
    def __init__(self):
        self._shift_api = Shifts()
        self._audit_log = logging.getLogger('audit')

    def POST(self, request, **kwargs):
        try:
            data = json.loads(request.body)
            name = self._valid_shift_name(data.get("name", "").strip())
            user_id = data.get("user_id", None)
            account_id = data.get("account_id", None)
            site_id = data.get("site_id", None)
            location_id = data.get("location_id", None)
            position_id = data.get("position_id", None)
            start_time = self._valid_time(data.get("start_time", "").strip())
            end_time = self._valid_time(data.get("end_time", "").strip())

            shift = self._shift_api.create_shift({'notes': name,
                                                  'start_time': start_time,
                                                  'end_time': end_time,
                                                  'user_id': user_id,
                                                  'account_id': account_id,
                                                  'site_id': site_id,
                                                  'location_id': location_id,
                                                  'position_id': position_id,
                                                  'published': True})

            shift_id = shift.shift_id

            self._audit_log.info('%s scheduled %s from %s to %s' % (
                request.user, name, start_time, end_time))

            return self.json_response({'shift_id': shift_id})
        except InvalidParamException as ex:
            return self.error_response(400, "%s" % ex)
        except Exception as ex:
            return self.error_response(500, "Unable to save shift: %s" % ex)

    def DELETE(self, request, **kwargs):
        try:
            shift_id = self._valid_shift_id(kwargs.get('shift_id'))

            self._shift_api.delete_shifts([shift_id])
            self._audit_log.info(
                '%s deleted shift %s' % (request.user, shift_id))
            return self.json_response({'deleted_shift_id': shift_id})
        except InvalidParamException as err:
            return self.error_response(400, "Invalid Parameter: %s" % err)

    def _valid_shift_id(self, shift_id):
        if re.match(r'^\d+$', shift_id):
            return shift_id

        raise InvalidParamException('missing shift id')

    def _valid_shift_name(self, name):
        if name and len(name):
            return name

        raise InvalidParamException('bad shift name')

    def _valid_time(self, time):
        if time and len(time):
            return time

        raise InvalidParamException('bad time value')



"""

"""
import logging

from restclients_core.exceptions import DataFailureException

from ems_wheniwork.utils import serviceorders_and_shifts
from . import RESTDispatch

logger = logging.getLogger(__name__)


class Schedule(RESTDispatch):
    def GET(self, request, **kwargs):
        try:
            events = serviceorders_and_shifts(request.GET)
        except DataFailureException as ex:
            return self.error_response(ex.status, str(ex))
        except Exception as ex:
            logger.exception(ex)
            return self.error_response(500, str(ex))

        return self.json_response(events)



import logging
import os
from time import localtime, strftime, time, tzset

from authz_group import Group
from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.http import HttpResponseRedirect
from django.shortcuts import render

logger = logging.getLogger(__name__)


@login_required
def index(request, template='ems_wheniwork/serviceorders.html'):
    user = request.user.username
    if not Group().is_member_of_group(user, settings.EMSTOOLS_SCHEDULER_GROUP):
        return HttpResponseRedirect("/")

    status_code = 200

    os.environ['TZ'] = 'America/Los_Angeles'
    tzset()

    context = {
        'todays_date': strftime("%Y-%m-%d"),
        'thirty_date': strftime("%Y-%m-%d", localtime(time() + 60*60*24*30)),
        'STATIC_URL': settings.STATIC_URL,
    }

    return render(request, template, context, status=status_code)



import logging
import re
from datetime import datetime, timedelta

import pytz
from dateutil import tz
from django.conf import settings
from ems_client.models import Status
from ems_client.service import Service
from uw_wheniwork.account import Accounts
from uw_wheniwork.locations import Locations
from uw_wheniwork.positions import Positions
from uw_wheniwork.shifts import Shifts
from uw_wheniwork.sites import Sites
from uw_wheniwork.users import Users


logger = logging.getLogger(__name__)


def serviceorders_and_shifts(params):
    search = {
        'start_date': params.get('StartDate'),
        'end_date': params.get('EndDate')
    }
    _ems = Service()

    event_shifts = []

    if not (search['start_date'] and search['end_date']):
        return None

    serviceorder_details = _ems.get_service_order_details(**search)
    bookings = _ems.get_bookings(**search)

    # build shifts
    assigned_serviceorders = {}
    for detail in serviceorder_details:
        if (detail.service_order_start_time is None or
                detail.service_order_end_time is None):
            continue

        # If this detail has an external reference,
        # then its parent service order is "assigned"
        if detail.resource_external_reference:
            assigned_serviceorders[detail.service_order_id] = detail
            continue

        for b in bookings:
            if b.id == detail.booking_id:
                break

        # Skip cancelled bookings
        if b.status_type_id == Status.STATUS_TYPE_CANCEL:
            continue

        event_shift = {
            'room': b.room_description,
            'roomname': b.dv_room,
            'name': None,
            'event_name': b.event_name,
            'resource_description': detail.resource_description,
            'start_time': detail.service_order_start_time.strftime("%-I:%M%p"),
            'end_time': detail.service_order_end_time.strftime("%-I:%M%p"),
            'hours': None,
            'service_order_id': detail.service_order_id,
            'booking_id': detail.booking_id,
            'building': b.dv_building,
            'room_code': b.room_code,
            'schedulable': True,
            'assigned_to': {
                'name': None,
                'netid': None,
            },
            'shift': {
                'name': None,
                'id': None,
                'user_id': None,
                'account_id': None,
                'site_id': None,
                'location_id': None,
                'position_id': None,
                'start_time': None,
                'end_time': None,
            },
        }

        emstz = pytz.timezone('America/Los_Angeles')
        start_time = emstz.localize(
            datetime.combine(detail.booking_date,
                             detail.service_order_start_time))
        end_time = emstz.localize(
            datetime.combine(detail.booking_date,
                             detail.service_order_end_time))
        if start_time > end_time:
            end_time += timedelta(days=1)
        start_utc = start_time.astimezone(tz.tzutc())
        end_utc = end_time.astimezone(tz.tzutc())

        event_shift['hours'] = re.sub(r'(\d+):(\d\d):\d\d', r'\1h\2m',
                                      str(end_utc - start_utc))
        event_shift['shift']['start_time'] = start_utc.isoformat()
        event_shift['shift']['end_time'] = end_utc.isoformat()
        event_shift['shift']['name'] = event_shift['name'] = \
            "%s - %s (Service Order %s) (%s-%s, %s)" % (
                event_shift['event_name'],
                event_shift['resource_description'],
                event_shift['service_order_id'],
                event_shift['start_time'],
                event_shift['end_time'],
                event_shift['hours'],
            )

        if event_shift:
            event_shifts.append(event_shift)

    mash_in_assigned_serviceorders(event_shifts, assigned_serviceorders)
    mash_in_wheniwork_shifts(event_shifts, params)

    return event_shifts


def mash_in_assigned_serviceorders(event_shifts, assigned_serviceorders):
    for e in event_shifts:
        id = e['service_order_id']
        if id in assigned_serviceorders:
            e['assigned_to']['name'] = \
                assigned_serviceorders[id].resource_description
            e['assigned_to']['netid'] = \
                assigned_serviceorders[id].resource_external_reference
            e['shift']['name'] = e['name'] = \
                "%s - %s (Service Order %s) (%s-%s, %s)" % (
                    e['event_name'],
                    e['assigned_to']['name'],
                    e['service_order_id'],
                    e['start_time'],
                    e['end_time'],
                    e['hours'],
                )


def mash_in_wheniwork_shifts(event_shifts, params):
    # mash in when i work shift schedule
    search = {
        'start': "%sT00:00:00" % params.get('StartDate'),
        'end': "%sT23:59:59" % params.get('EndDate'),
        'include_allopen': True,
    }

    # Populate account record
    account_id = Accounts().get_account().account_id

    shifts = Shifts().get_shifts(search)
    for shift in shifts if shifts else []:
        for e in event_shifts:
            if "Service Order %s" % e['service_order_id'] in shift.notes:
                e['shift']['name'] = shift.notes
                e['shift']['id'] = shift.shift_id
                e['shift']['user_id'] = shift.user_id
                e['shift']['account_id'] = shift.account_id
                e['shift']['site_id'] = shift.site_id
                e['shift']['location_id'] = shift.location_id
                e['shift']['position_id'] = shift.position_id

                # actual shift start and end
                start_utc = shift.start_time.astimezone(pytz.utc)
                end_utc = shift.end_time.astimezone(pytz.utc)
                e['shift']['start'] = start_utc.isoformat()
                e['shift']['end'] = end_utc.isoformat()

    locations = Locations().get_locations()
    for l in locations:
        if l.name == settings.EMS_WHENIWORK_SCHEDULE_LOCATION:
            location_id = l.location_id
            break

    positions = Positions().get_positions()
    for p in positions:
        if p.name == settings.EMS_WHENIWORK_SCHEDULE_POSITION:
            position_id = p.position_id
            break

    users = Users().get_users({'location_id': location_id})

    _sites = Sites()
    sites = _sites.get_sites()

    for e in event_shifts:
        if not e['shift']['account_id']:
            e['shift']['account_id'] = account_id

        if location_id and not e['shift']['location_id']:
            e['shift']['location_id'] = location_id

        if position_id and not e['shift']['position_id']:
            e['shift']['position_id'] = position_id

        if not e['shift']['site_id']:
            for s in sites:
                if e['room'] in s.name:
                    e['shift']['site_id'] = s.site_id
                    break

        # create site if not found
        if not e['shift']['site_id']:
            new_site = {
                'location_id': location_id,
                'name': e['room'],
                # 'address': "%s %s, University of Washington" %
                #            (e['room_code'], e['building'])
            }
            if e['roomname'] != e['room']:
                new_site['name'] = new_site['name'] + " (%s)" % e['roomname']

            s = _sites.create_site(new_site)
            sites.append(s)
            e['shift']['site_id'] = s.site_id

        if not e['shift']['user_id']:
            for u in users:
                if u.employee_code == e['assigned_to']['netid']:
                    e['shift']['user_id'] = u.user_id
                    break
        if not e['shift']['user_id']:
            e['schedulable'] = False
